<?php

namespace Drupal\move_file\Services;

use Drupal\Core\Config\ConfigFactory;
use Drupal\move_file\Entity\DirectoryEntity;
use Drupal\node\Entity\Node;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Move files referenced by a node.
 *
 * @package Drupal\move_file
 */
class MoveFileService {

  /**
   * Configurations.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected $config;

  /**
   * Constructor.
   *
   * @param \Drupal\Core\Config\ConfigFactory $config_factory
   *   The config factory.
   */
  public function __construct(ConfigFactory $config_factory) {
    $this->config = $config_factory->get('move_file.settings');
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory')
    );
  }

  /**
   * Move files referenced by a node.
   *
   * @param \Drupal\node\Entity\Node $node
   *   The files referenced by this node will be moved.
   */
  public function move(Node $node) {
    // Get activated content types from the config object.
    $activated_content_types = [];
    $activated_content_settings = [];
    foreach ($this->config->get('content_types') as $content_type_name => $content_type_fields) {
      $activated_content_types[] = $content_type_name;
      $activated_content_settings[$content_type_name] = $content_type_fields;
    }

    $node_type_name = $node->getType();
    if (in_array($node_type_name, $activated_content_types)) {

      // Get all selected taxonomy term ids.
      $vocabulary_field = $activated_content_settings[$node_type_name]['vocabulary_field'];
      $selected_term_ids = [];
      foreach ($node->{$vocabulary_field} as $selected_term) {
        $selected_term_ids[] = $selected_term->getValue()['target_id'];
      }

      // Get target directory.
      $target_directory = FALSE;
      foreach (DirectoryEntity::loadMultiple() as $directory) {
        if (in_array($directory->term_id, $selected_term_ids)) {
          $target_directory = $directory;
          break 1;
        }
      }

      // Define the target path.
      $file_target_path = FALSE;
      if ($target_directory) {
        // Get the directory entity.
        $file_scheme = $target_directory->private ? 'private' : 'public';
        $file_target_path = $file_scheme . ':/' . $target_directory->path . '/';
      }

      // Move all files from all activated fields to the target path.
      if ($file_target_path) {
        foreach ($activated_content_settings[$node_type_name]['file_field'] as $activated_file_field) {
          foreach ($node->get($activated_file_field) as $fileRef) {
            // The following lines are resulting in: "File::load calls should be
            // avoided in classes, use dependency injection instead."
            // $fileId = $fileRef->getValue()['target_id'];
            // $file = File::load($fileId);
            // The solution is a bit strange, see:
            // https://www.drupal.org/project/drupal/issues/2785425
            $file = $fileRef->view()["#file"];
            file_move($file, $file_target_path . $file->getFilename());
          }
        }
      }
    }
  }

}
